@@ -37,10 +37,10 @@
 
 
 static void
-clearlooks_draw_glossy_gradient (cairo_t *cr,
-                              double x, double y, int width, int height,
-                              const CairoColor *color, boolean disabled,
-                              gboolean radius, CairoCorners corners)
+clearlooks_draw_glossy_gradient (cairo_t         *cr,
+                                double x, double y, int width, int height,
+                                const CairoColor *color,
+                                gboolean disabled, gboolean radius, CairoCorners corners)
 {
 	CairoColor a, b, c, d;
 	cairo_pattern_t *pt;
@@ -64,25 +64,29 @@ clearlooks_draw_glossy_gradient (cairo_t *cr,
 }
 
 static void
-clearlooks_set_mixed_color (cairo_t *cr, const CairoColor *color1, const CairoColor *color2, double mix_factor)
+clearlooks_set_mixed_color (cairo_t          *cr, 
+                            const CairoColor *color1, 
+                            const CairoColor *color2, 
+                            gdouble mix_factor)
 {
 	CairoColor composite;
 
 	ge_mix_color (color1, color2, mix_factor, &composite);
-	ge_cairo_set_color (cr, &composite);	
+	ge_cairo_set_color (cr, &composite);
 }
 
 static void
-clearlooks_glossy_draw_inset (cairo_t *cr, CairoColor bg_color, 
-                       double x, double y, double w, double h, 
-                       double radius, uint8 corners)
+clearlooks_glossy_draw_inset (cairo_t          *cr, 
+                              const CairoColor *bg_color,
+                              double x, double y, double w, double h,
+                              double radius, uint8 corners)
 {
 	CairoColor shadow;
 	CairoColor highlight;
 
 	/* not really sure of shading ratios... we will think */
-	ge_shade_color (&bg_color, 0.93, &shadow);
-	ge_shade_color (&bg_color, 1.07, &highlight);
+	ge_shade_color (bg_color, 0.93, &shadow);
+	ge_shade_color (bg_color, 1.07, &highlight);
 
 	/* highlight */
 	cairo_move_to (cr, x + w + (radius * -0.2928932188), y - (radius * -0.2928932188)); /* 0.2928932... 1-sqrt(2)/2 gives middle of curve */
@@ -128,16 +132,17 @@ clearlooks_glossy_draw_inset (cairo_t *cr, CairoColor bg_color,
 }
 
 static void
-clearlooks_glossy_draw_light_inset (cairo_t *cr, CairoColor bg_color, 
-                       double x, double y, double w, double h, 
-                       double radius, uint8 corners)
+clearlooks_glossy_draw_light_inset (cairo_t          *cr, 
+                                    const CairoColor *bg_color, 
+                                    double x, double y, double w, double h, 
+                                    double radius, uint8 corners)
 {
 	CairoColor shadow;
 	CairoColor highlight;
 
 	/* not really sure of shading ratios... we will think */
-	ge_shade_color (&bg_color, 0.95, &shadow);
-	ge_shade_color (&bg_color, 1.05, &highlight);
+	ge_shade_color (bg_color, 0.95, &shadow);
+	ge_shade_color (bg_color, 1.05, &highlight);
 
 	/* highlight */
 	cairo_move_to (cr, x + w + (radius * -0.2928932188), y - (radius * -0.2928932188)); /* 0.2928932... 1-sqrt(2)/2 gives middle of curve */
@@ -183,9 +188,10 @@ clearlooks_glossy_draw_light_inset (cairo_t *cr, CairoColor bg_color,
 }
 
 static void
-clearlooks_glossy_draw_highlight_and_shade (cairo_t *cr, CairoColor bg_color,
-                                     const ShadowParameters *params,
-                                     int width, int height, gdouble radius)
+clearlooks_glossy_draw_highlight_and_shade (cairo_t          *cr, 
+                                            const CairoColor *bg_color,
+                                            const ShadowParameters *params,
+                                            int width, int height, gdouble radius)
 {
 	CairoColor shadow;
 	CairoColor highlight;
@@ -194,8 +200,8 @@ clearlooks_glossy_draw_highlight_and_shade (cairo_t *cr, CairoColor bg_color,
 	double y = 1.0;
 
 	/* not really sure of shading ratios... we will think */
-	ge_shade_color (&bg_color, 0.8, &shadow);
-	ge_shade_color (&bg_color, 1.2, &highlight);
+	ge_shade_color (bg_color, 0.8, &shadow);
+	ge_shade_color (bg_color, 1.2, &highlight);
 	
 	cairo_save (cr);
 	
@@ -216,7 +222,7 @@ clearlooks_glossy_draw_highlight_and_shade (cairo_t *cr, CairoColor bg_color,
 		cairo_set_source_rgba (cr, highlight.r, highlight.g, highlight.b, 0.5);
 	else
 		cairo_set_source_rgba (cr, shadow.r, shadow.g, shadow.b, 0.5);
-		
+	
 	cairo_stroke (cr);
 	
 	/* Bottom/Right highlight -- this includes the corners */
@@ -277,7 +283,7 @@ clearlooks_glossy_draw_button (cairo_t *cr,
 			radius = MIN (params->radius, MIN ((width - 2.0 - 2*xoffset) / 2.0 - 1.0, (height - 2.0 - 2*yoffset) / 2.0 - 1.0));
 
 			ge_cairo_rounded_rectangle (cr, 0, 0, width-1, height-1, radius+1, params->corners);
-			ge_shade_color (&params->parentbg, 0.96, &glow);			
+			ge_shade_color (&params->parentbg, 0.96, &glow);
 			ge_cairo_set_color (cr, &glow);
 			cairo_stroke (cr);
 
@@ -290,10 +296,10 @@ clearlooks_glossy_draw_button (cairo_t *cr,
 		/* if (!(params->enable_glow && !params->active && !params->disabled)) */
 		if (!(params->prelight && params->enable_glow && !params->active))
 			if (!(params->disabled))
-				params->style_functions->draw_inset (cr, params->parentbg, 0, 0, width-1, height-1, params->radius+1, params->corners);
+				params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, width-1, height-1, params->radius+1, params->corners);
 			else
 				/*Draw a lighter inset */
-				clearlooks_glossy_draw_light_inset (cr, params->parentbg, 0, 0, width-1, height-1, params->radius+1, params->corners);
+				clearlooks_glossy_draw_light_inset (cr, &params->parentbg, 0, 0, width-1, height-1, params->radius+1, params->corners);
 		cairo_translate (cr, -0.5, -0.5);
 	}
 
@@ -390,7 +396,7 @@ clearlooks_glossy_draw_progressbar_trough (cairo_t *cr,
 	/* Fill with bg color */
 	ge_cairo_set_color (cr, &colors->bg[params->state_type]);
 	
-	cairo_rectangle (cr, x, y, width, height);	
+	cairo_rectangle (cr, x, y, width, height);
 	cairo_fill (cr);
 
 	/* Create trough box */
@@ -412,8 +418,8 @@ clearlooks_glossy_draw_progressbar_trough (cairo_t *cr,
 	/* Top shadow */
 	cairo_rectangle (cr, x+1, y+1, width-2, 4);
 	pattern = cairo_pattern_create_linear (x, y, x, y+4);
-	cairo_pattern_add_color_stop_rgba (pattern, 0.0, shadow.r, shadow.g, shadow.b, 0.3);	
-	cairo_pattern_add_color_stop_rgba (pattern, 1.0, shadow.r, shadow.g, shadow.b, 0.);	
+	cairo_pattern_add_color_stop_rgba (pattern, 0.0, shadow.r, shadow.g, shadow.b, 0.3);
+	cairo_pattern_add_color_stop_rgba (pattern, 1.0, shadow.r, shadow.g, shadow.b, 0.);
 	cairo_set_source (cr, pattern);
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
@@ -421,8 +427,8 @@ clearlooks_glossy_draw_progressbar_trough (cairo_t *cr,
 	/* Left shadow */
 	cairo_rectangle (cr, x+1, y+1, 4, height-2);
 	pattern = cairo_pattern_create_linear (x, y, x+4, y);
-	cairo_pattern_add_color_stop_rgba (pattern, 0.0, shadow.r, shadow.g, shadow.b, 0.3);	
-	cairo_pattern_add_color_stop_rgba (pattern, 1.0, shadow.r, shadow.g, shadow.b, 0.);	
+	cairo_pattern_add_color_stop_rgba (pattern, 0.0, shadow.r, shadow.g, shadow.b, 0.3);
+	cairo_pattern_add_color_stop_rgba (pattern, 1.0, shadow.r, shadow.g, shadow.b, 0.);
 	cairo_set_source (cr, pattern);
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
@@ -604,13 +610,13 @@ clearlooks_glossy_scale_draw_gradient (cairo_t *cr,
 	cairo_pattern_add_color_stop_rgb (pattern, 0.0, c1->r, c1->g, c1->b);
 	cairo_pattern_add_color_stop_rgb (pattern, 1.0, c2->r, c2->g, c2->b);
 
-	cairo_rectangle (cr, x+0.5, y+0.5, width-1, height-1);	
+	cairo_rectangle (cr, x+0.5, y+0.5, width-1, height-1);
 	cairo_set_source (cr, pattern);
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
 	
 	clearlooks_set_mixed_color (cr, c3, c1, 0.3);
-	ge_cairo_stroke_rectangle (cr, x, y, width, height);	
+	ge_cairo_stroke_rectangle (cr, x, y, width, height);
 }
 
 #define TROUGH_SIZE 6
@@ -643,19 +649,20 @@ clearlooks_glossy_draw_scale_trough (cairo_t *cr,
 
 	cairo_set_line_width (cr, 1.0);
 	cairo_translate (cr, translate_x, translate_y);
-	
-	params->style_functions->draw_inset (cr, params->parentbg, 0, 0, trough_width+2, trough_height+2, 0, 0);
+
+	if (!slider->fill_level)
+		params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, trough_width+2, trough_height+2, 0, 0);
 	
 	cairo_translate (cr, 1, 1);
 	
-	if (!slider->lower)
+	if (!slider->lower && !slider->fill_level)
 		clearlooks_glossy_scale_draw_gradient (cr, &colors->shade[3], /* top */
 		                                    &colors->shade[2], /* bottom */
 		                                    &colors->shade[6], /* border */
 		                                    0, 0, trough_width, trough_height,
 		                                    slider->horizontal);
 	else
-		clearlooks_glossy_scale_draw_gradient (cr, &colors->spot[1], /* top    */
+		clearlooks_glossy_scale_draw_gradient (cr, &colors->spot[1], /* top */
 		                                    &colors->spot[0], /* bottom */
 		                                    &colors->spot[2], /* border */
 		                                    0, 0, trough_width, trough_height,
@@ -688,7 +695,7 @@ clearlooks_glossy_draw_tab (cairo_t *cr,
 	cairo_clip           (cr);
 	cairo_new_path       (cr);
 
-	/* Translate and set line width */	
+	/* Translate and set line width */
 	cairo_set_line_width (cr, 1.0);
 	cairo_translate      (cr, x+0.5, y+0.5);
 
@@ -733,7 +740,7 @@ clearlooks_glossy_draw_tab (cairo_t *cr,
 		shadow.shadow  = CL_SHADOW_OUT;
 		shadow.corners = params->corners;
 		
-		clearlooks_glossy_draw_highlight_and_shade (cr, colors->bg[0], &shadow,
+		clearlooks_glossy_draw_highlight_and_shade (cr, &colors->bg[0], &shadow,
 		                                     width,
 		                                     height, radius);
 	}
@@ -742,10 +749,10 @@ clearlooks_glossy_draw_tab (cairo_t *cr,
 	{
 		CairoColor shadow, hilight, f1, f2;
 
-		pattern = cairo_pattern_create_linear ( tab->gap_side == CL_GAP_LEFT   ? width-1  : 0,
-		                                        tab->gap_side == CL_GAP_TOP    ? height-2 : 1,
-		                                        tab->gap_side == CL_GAP_RIGHT  ? width    : 0,
-		                                        tab->gap_side == CL_GAP_BOTTOM ? height   : 0 );
+		pattern = cairo_pattern_create_linear (tab->gap_side == CL_GAP_LEFT   ? width-1  : 0,
+		                                       tab->gap_side == CL_GAP_TOP    ? height-2 : 1,
+		                                       tab->gap_side == CL_GAP_RIGHT  ? width    : 0,
+		                                       tab->gap_side == CL_GAP_BOTTOM ? height   : 0);
 
 		ge_cairo_rounded_rectangle (cr, 0, 0, width-1, height-1, radius, params->corners);
 		
@@ -754,8 +761,8 @@ clearlooks_glossy_draw_tab (cairo_t *cr,
 		ge_shade_color (fill, 1.12, &f1);
 		ge_shade_color (fill, 1.06, &f2);
 
-		cairo_pattern_add_color_stop_rgb (pattern, 0.0,        hilight.r, hilight.g, hilight.b);     
-		cairo_pattern_add_color_stop_rgb (pattern, 1.0/height, hilight.r, hilight.g, hilight.b); 
+		cairo_pattern_add_color_stop_rgb (pattern, 0.0,        hilight.r, hilight.g, hilight.b);
+		cairo_pattern_add_color_stop_rgb (pattern, 1.0/height, hilight.r, hilight.g, hilight.b);
 		cairo_pattern_add_color_stop_rgb (pattern, 1.0/height, f1.r, f1.g, f1.b);
 		cairo_pattern_add_color_stop_rgb (pattern, 0.45,        f2.r, f2.g, f2.b);
 		cairo_pattern_add_color_stop_rgb (pattern, 0.45,        fill->r, fill->g, fill->b);
@@ -767,10 +774,10 @@ clearlooks_glossy_draw_tab (cairo_t *cr,
 	else
 	{
 		/* Draw shade */
-		pattern = cairo_pattern_create_linear ( tab->gap_side == CL_GAP_LEFT   ? width-2  : 0,
-		                                        tab->gap_side == CL_GAP_TOP    ? height-2 : 0,
-		                                        tab->gap_side == CL_GAP_RIGHT  ? width    : 0,
-		                                        tab->gap_side == CL_GAP_BOTTOM ? height   : 0 );
+		pattern = cairo_pattern_create_linear (tab->gap_side == CL_GAP_LEFT   ? width-2  : 0,
+		                                       tab->gap_side == CL_GAP_TOP    ? height-2 : 0,
+		                                       tab->gap_side == CL_GAP_RIGHT  ? width    : 0,
+		                                       tab->gap_side == CL_GAP_BOTTOM ? height   : 0);
 	
 		ge_cairo_rounded_rectangle (cr, 0, 0, width-1, height-1, radius, params->corners);
 		
@@ -786,15 +793,15 @@ clearlooks_glossy_draw_tab (cairo_t *cr,
 	
 	if (params->active)
 	{
-		ge_cairo_set_color (cr, border);	
+		ge_cairo_set_color (cr, border);
 		cairo_stroke (cr);
 	}
 	else
 	{
-		pattern = cairo_pattern_create_linear ( tab->gap_side == CL_GAP_LEFT   ? width-2  : 2,
-		                                        tab->gap_side == CL_GAP_TOP    ? height-2 : 2,
-		                                        tab->gap_side == CL_GAP_RIGHT  ? width    : 2,
-		                                        tab->gap_side == CL_GAP_BOTTOM ? height   : 2 );
+		pattern = cairo_pattern_create_linear (tab->gap_side == CL_GAP_LEFT   ? width-2  : 2,
+		                                       tab->gap_side == CL_GAP_TOP    ? height-2 : 2,
+		                                       tab->gap_side == CL_GAP_RIGHT  ? width    : 2,
+		                                       tab->gap_side == CL_GAP_BOTTOM ? height   : 2);
 		
 		cairo_pattern_add_color_stop_rgb (pattern, 0.0, stripe_border->r, stripe_border->g, stripe_border->b);
 		cairo_pattern_add_color_stop_rgb (pattern, 0.8, border->r,        border->g,        border->b);
@@ -914,8 +921,8 @@ clearlooks_glossy_draw_scrollbar_stepper (cairo_t *cr,
 				
 	fill = colors->bg[widget->state_type];
 	ge_shade_color(&fill, 1.16, &s1);
-	ge_shade_color(&fill, 1.08, &s2); 
-	ge_shade_color(&fill, 1.08, &s4); 
+	ge_shade_color(&fill, 1.08, &s2);
+	ge_shade_color(&fill, 1.08, &s4);
 	
 	cairo_pattern_add_color_stop_rgb(pattern, 0,    s1.r, s1.g, s1.b);
 	cairo_pattern_add_color_stop_rgb(pattern, 0.5,	s2.r, s2.g, s2.b);
@@ -976,7 +983,7 @@ clearlooks_glossy_draw_scrollbar_slider (cairo_t *cr,
 	if (!scrollbar->horizontal)
 		ge_cairo_exchange_axis (cr, &x, &y, &width, &height);
 
-	cairo_translate (cr, x, y);	
+	cairo_translate (cr, x, y);
 
 	if (widget->prelight)
 		ge_shade_color (&fill, 1.1, &fill);
@@ -992,19 +999,20 @@ clearlooks_glossy_draw_scrollbar_slider (cairo_t *cr,
 	cairo_pattern_add_color_stop_rgb (pattern, 0,   shade1.r, shade1.g, shade1.b);
 	cairo_pattern_add_color_stop_rgb (pattern, 0.5,	shade2.r, shade2.g, shade2.b);
 	cairo_pattern_add_color_stop_rgb (pattern, 0.5, 	fill.r,  fill.g,  fill.b);
-	cairo_pattern_add_color_stop_rgb (pattern, 1,	shade3.r, shade3.g, shade3.b);	
+	cairo_pattern_add_color_stop_rgb (pattern, 1,	shade3.r, shade3.g, shade3.b);
 	cairo_rectangle (cr, 1, 1, width-2, height-2);
 	cairo_set_source (cr, pattern);
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
 	
-	if (scrollbar->has_color) {
+	if (scrollbar->has_color) 
+	{
 		cairo_set_source_rgba (cr, hilight.r, hilight.g, hilight.b, 0.5);
 		ge_cairo_stroke_rectangle (cr, 1.5, 1.5, width-3, height-3);
 	}
 
 	clearlooks_set_mixed_color (cr, border, &fill, scrollbar->has_color? 0.4 : 0.2);
-	ge_cairo_stroke_rectangle (cr, 0.5, 0.5, width-1, height-1);	
+	ge_cairo_stroke_rectangle (cr, 0.5, 0.5, width-1, height-1);
 }
 
 static void
@@ -1077,6 +1085,65 @@ clearlooks_glossy_draw_list_view_header (cairo_t *cr,
 	}
 }
 
+static void 
+clearlooks_glossy_draw_toolbar (cairo_t *cr,
+                         const ClearlooksColors          *colors,
+                         const WidgetParameters          *widget,
+                         const ToolbarParameters         *toolbar,
+                         int x, int y, int width, int height)
+{
+	const CairoColor *fill  = &colors->bg[GTK_STATE_NORMAL];
+	const CairoColor *dark  = &colors->shade[3];
+	CairoColor light;
+	ge_shade_color (fill, 1.1, &light);
+	
+	cairo_set_line_width (cr, 1.0);
+	cairo_translate (cr, x, y);
+	
+	if (toolbar->style == 1) /* Enable Extra features */
+	{ 
+		cairo_pattern_t *pattern;
+		CairoColor shade1, shade2, shade3;
+		
+		ge_shade_color (fill, 1.08, &shade1);
+		ge_shade_color (fill, 1.04, &shade2);
+		ge_shade_color (fill, 1.04, &shade3);
+
+		/* Draw the fill */
+		pattern = cairo_pattern_create_linear (0, 0, 0, height);
+		cairo_pattern_add_color_stop_rgb (pattern, 0.0, shade1.r, shade1.g, shade1.b);
+		cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade2.r, shade2.g, shade2.b);
+		cairo_pattern_add_color_stop_rgb (pattern, 0.5, fill->r, fill->g, fill->b);
+		cairo_pattern_add_color_stop_rgb (pattern, 1.0, shade3.r, shade3.g, shade3.b);
+
+		cairo_set_source (cr, pattern);
+		cairo_rectangle (cr, 0, 0, width, height);
+		cairo_fill (cr);
+
+		cairo_pattern_destroy (pattern);
+	}
+	else /* Flat */
+	{ 
+		ge_cairo_set_color (cr, fill);
+		cairo_paint (cr);
+
+		if (!toolbar->topmost) 
+		{
+			/* Draw highlight */
+			cairo_move_to       (cr, 0, 0.5);
+			cairo_line_to       (cr, width-1, 0.5);
+			ge_cairo_set_color  (cr, &light);
+			cairo_stroke        (cr);
+		}	
+	}
+
+	/* Draw shadow */
+	cairo_move_to       (cr, 0, height-0.5);
+	cairo_line_to       (cr, width-1, height-0.5);
+	ge_cairo_set_color  (cr, dark);
+	cairo_stroke        (cr);
+}
+
 static void
 clearlooks_glossy_draw_menuitem (cairo_t                   *cr,
                                  const ClearlooksColors    *colors,
@@ -1096,9 +1163,9 @@ clearlooks_glossy_draw_menuitem (cairo_t                   *cr,
 
 static void
 clearlooks_glossy_draw_menubaritem (cairo_t                   *cr,
-                                 const ClearlooksColors    *colors,
-                                 const WidgetParameters    *params,
-                                 int x, int y, int width, int height)
+                                    const ClearlooksColors    *colors,
+                                    const WidgetParameters    *params,
+                                    int x, int y, int width, int height)
 {
 	cairo_save (cr);
 	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1, height, params->radius, params->corners);
@@ -1172,14 +1239,14 @@ clearlooks_glossy_draw_radiobutton (cairo_t *cr,
 	cairo_translate (cr, x, y);
 	
 	cairo_set_line_width (cr, 2);
-	cairo_arc       (cr, 7, 7, 6, 0, G_PI*2);	
+	cairo_arc       (cr, 7, 7, 6, 0, G_PI*2);
 	cairo_set_source (cr, pt);
 	cairo_stroke (cr);
 	cairo_pattern_destroy (pt);
 
 	cairo_set_line_width (cr, 1);
 
-	cairo_arc       (cr, 7, 7, 5.5, 0, G_PI*2);	
+	cairo_arc       (cr, 7, 7, 5.5, 0, G_PI*2);
 	
 	if (!widget->disabled)
 	{
@@ -1253,7 +1320,7 @@ clearlooks_glossy_draw_checkbox (cairo_t *cr,
 	
 	if (widget->xthickness > 2 && widget->ythickness > 2)
 	{
-		widget->style_functions->draw_inset (cr, widget->parentbg, 0.5, 0.5, 
+		widget->style_functions->draw_inset (cr, &widget->parentbg, 0.5, 0.5, 
                                            width-1, height-1, (widget->radius > 0)? 1 : 0, CR_CORNER_ALL);
 		
 		/* Draw the rectangle for the checkbox itself */
@@ -1268,10 +1335,10 @@ clearlooks_glossy_draw_checkbox (cairo_t *cr,
 	}
 	
 	if (!widget->disabled)
-	{		
+	{
 		if (widget->prelight)
 			clearlooks_set_mixed_color (cr, &colors->base[0], &colors->spot[1], 0.5);
-		else		
+		else
 			ge_cairo_set_color (cr, &colors->base[0]);
 		cairo_fill_preserve (cr);
 	}
@@ -1318,6 +1385,7 @@ clearlooks_register_style_glossy (ClearlooksStyleFunctions *functions)
 	functions->draw_scrollbar_stepper  = clearlooks_glossy_draw_scrollbar_stepper;
 	functions->draw_scrollbar_slider   = clearlooks_glossy_draw_scrollbar_slider;
 	functions->draw_list_view_header   = clearlooks_glossy_draw_list_view_header;
+	functions->draw_toolbar            = clearlooks_glossy_draw_toolbar;
 	functions->draw_menuitem           = clearlooks_glossy_draw_menuitem;
 	functions->draw_menubaritem        = clearlooks_glossy_draw_menubaritem;
 	functions->draw_selected_cell      = clearlooks_glossy_draw_selected_cell;
