@@ -51,7 +51,7 @@ clearlooks_get_parent_bg (const GtkWidget *widget, CairoColor *color)
 	
 	parent = widget->parent;
 	
-	while (parent && GTK_WIDGET_NO_WINDOW (parent))
+	while (parent && GTK_WIDGET_NO_WINDOW (parent) && !((GTK_IS_NOTEBOOK (parent)) || (GTK_IS_TOOLBAR (parent))))
 		parent = parent->parent;
 
 	if (parent == NULL)
@@ -173,3 +173,20 @@ clearlooks_scrollbar_get_junction (GtkWidget    *widget)
 	
 	return junction;
 }
+
+void
+clearlooks_set_toolbar_parameters (ToolbarParameters *toolbar, GtkWidget *widget, GdkWindow *window, gint x, gint y)
+{
+	toolbar->topmost = FALSE;
+
+	if (x == 0 && y == 0) {
+		if (widget && widget->allocation.x == 0 && widget->allocation.y == 0)
+		{
+			if (widget->window == window && GE_IS_TOOLBAR (widget))
+			{
+				toolbar->topmost = TRUE;
+			}
+		}
+	}
+}
+
